require_dependency 'versions_controller'

module UsabilityImprovementVersionsControllerPatch
  
    def self.included(base) # :nodoc:
      
      base.send(:include, InstanceMethods)
      base.class_eval do
        helper :sort
        include SortHelper
        helper :queries
        include QueriesHelper
        helper :issues
        include IssuesHelper
        unloadable
        before_filter :find_optional_project, :only => [:index]
        alias_method_chain :index, :issue_list
      end
        
    end
   
    module InstanceMethods
      
      def index_with_issue_list
        
        @trackers = @project.trackers.find(:all, :order => 'position')
        retrieve_selected_tracker_ids(@trackers, @trackers.select {|t| t.is_in_roadmap?})
        @with_subprojects = params[:with_subprojects].nil? ? Setting.display_subprojects_issues? : (params[:with_subprojects] == '1')
        project_ids = @with_subprojects ? @project.self_and_descendants.collect(&:id) : [@project.id]
        @selected_trackers = Tracker.find(params[:tracker_ids]) unless params[:tracker_ids].blank?

        @versions = @project.shared_versions || []
        @versions += @project.rolled_up_versions.visible if @with_subprojects
        @versions = @versions.uniq.sort
        unless params[:completed]
          @completed_versions = @versions.select {|version| version.closed? || version.completed? }
          @versions -= @completed_versions
        end

        @versions.reject! {|version| !project_ids.include?(version.project_id) && @issues_by_version[version].blank?}
        
        params[:group_by] = 'fixed_version'
        session[:query][:group_by] = 'fixed_version' if session[:query]
        retrieve_query
        sort_init(@query.sort_criteria.empty? ? [['id', 'desc']] : @query.sort_criteria)
        sort_update(@query.sortable_columns)
        
        if @query.valid?
          @issues = @query.issues(:include => [:assigned_to, :tracker, :priority, :category, :fixed_version],
                                  :order => sort_clause)
          if @issues.any?
            @issues = @issues.select {|i| @versions.map(&:id).include?(i.fixed_version_id)} unless @versions.blank?
            @issues = @issues.select {|i| @selected_trackers.map(&:id).include?(i.tracker_id)} unless @selected_trackers.blank?
            @total_estimated_hours = 0
            @issues.each {|i| @total_estimated_hours += i.estimated_hours if i.leaf?}
            @total_spent_hours = 0
            @issues.each {|i| @total_spent_hours += i.spent_hours}
          end
          @issue_count_by_group = @query.issue_count_by_group
        end
        
        respond_to do |format|
          format.html { render(:template => 'versions/index', :layout => !request.xhr?) }
          format.any(:atom, :csv, :pdf) { render(:nothing => true) }
          format.api  { @versions = @project.shared_versions.all }
        end

      rescue ActiveRecord::RecordNotFound
        render_404
      end
      
    end
    
end
 
VersionsController.send(:include, UsabilityImprovementVersionsControllerPatch)
 
